@@ -1,5 +1,5 @@
 import math
-from collections.abc import Callable, Mapping, Sequence, Set
+from collections.abc import Callable, Mapping, MutableMapping, MutableSequence, Sequence, Set
 from typing import Any, SupportsFloat, SupportsInt
 
 from iker.common.utils.numutils import is_normal_real
@@ -15,7 +15,12 @@ __all__ = [
     "JsonArrayCompatible",
     "JsonObjectCompatible",
     "JsonTypeCompatible",
+    "NodeAsterisk",
+    "Node",
     "NodePath",
+    "asterisk",
+    "json_get",
+    "json_set",
     "json_traverse",
     "json_reformat",
     "json_sanitize",
@@ -34,7 +39,103 @@ JsonObjectCompatible = Mapping[JsonKeyCompatible, "JsonTypeCompatible"]
 JsonArrayCompatible = Sequence["JsonTypeCompatible"]
 JsonTypeCompatible = JsonValueCompatible | JsonObjectCompatible | JsonArrayCompatible
 
-NodePath = list[str | int]
+
+class NodeAsterisk(object):
+    pass
+
+
+Node = str | int | NodeAsterisk
+NodePath = list[Node]
+
+asterisk = NodeAsterisk()
+
+
+def json_get(
+    obj: JsonTypeCompatible,
+    *nodes: Node,
+    fallback: JsonTypeCompatible | None = None,
+) -> JsonTypeCompatible | None:
+    if obj is None or isinstance(obj, (JsonValue | JsonValueCompatible)):
+        if len(nodes) == 0:
+            return obj
+        return fallback
+
+    if isinstance(obj, Mapping):
+        if len(nodes) == 0:
+            return obj
+        head, *rest = nodes
+        if not isinstance(head, str):
+            return fallback
+        return json_get(obj[head], *rest) if head in obj else fallback
+
+    if isinstance(obj, Sequence):
+        if len(nodes) == 0:
+            return obj
+        head, *rest = nodes
+        if not isinstance(head, int):
+            return fallback
+        return json_get(obj[head], *rest) if -len(obj) <= head < len(obj) else fallback
+
+    raise ValueError(f"unregistered type '{type(obj)}' encountered")
+
+
+def json_set(obj: JsonTypeCompatible, value: JsonTypeCompatible, *nodes: Node) -> bool:
+    if len(nodes) == 0:
+        return False
+
+    if isinstance(obj, Mapping):
+        if not isinstance(obj, MutableMapping):
+            raise ValueError(f"immutable mapping type '{type(obj)}' encountered")
+
+        head, *rest = nodes
+        if not isinstance(head, str):
+            return False
+        if head not in obj:
+            obj[head] = None
+
+        if len(rest) == 0:
+            obj[head] = value
+            return True
+        rest_head, *rest_rest = rest
+        if isinstance(rest_head, int) or isinstance(rest_head, NodeAsterisk):
+            if not isinstance(obj[head], MutableSequence):
+                obj[head] = []
+            return json_set(obj[head], value, *rest)
+        if isinstance(rest_head, str):
+            if not isinstance(obj[head], MutableMapping):
+                obj[head] = {}
+            return json_set(obj[head], value, *rest)
+
+    if isinstance(obj, Sequence):
+        if not isinstance(obj, MutableSequence):
+            raise ValueError(f"immutable sequence type '{type(obj)}' encountered")
+
+        head, *rest = nodes
+        if not isinstance(head, int) and not isinstance(head, NodeAsterisk):
+            return False
+        if isinstance(head, NodeAsterisk):
+            head = len(obj)
+        while not -len(obj) <= head < len(obj):
+            if head < 0:
+                obj.reverse()
+            obj.append(None)
+            if head < 0:
+                obj.reverse()
+
+        if len(rest) == 0:
+            obj[head] = value
+            return True
+        rest_head, *rest_rest = rest
+        if isinstance(rest_head, int) or isinstance(rest_head, NodeAsterisk):
+            if not isinstance(obj[head], MutableSequence):
+                obj[head] = []
+            return json_set(obj[head], value, *rest)
+        if isinstance(rest_head, str):
+            if not isinstance(obj[head], MutableMapping):
+                obj[head] = {}
+            return json_set(obj[head], value, *rest)
+
+    return False
 
 
 def default_key_visitor(node_path: NodePath, key: JsonKeyCompatible) -> JsonKey:
@@ -151,8 +252,8 @@ def json_sanitize(obj: Any, *, str_inf_nan: bool = True, str_unregistered: bool
 
 
 def json_compare(
-    a: JsonType,
-    b: JsonType,
+    a: JsonTypeCompatible,
+    b: JsonTypeCompatible,
     *,
     int_strict: bool = False,
     float_tol: float = 1e-5,
@@ -161,48 +262,56 @@ def json_compare(
 ) -> bool:
     if a is None or b is None:
         return a is None and b is None
-    if isinstance(a, (dict, list, str, bool)):
+
+    if isinstance(a, (str, bool)):
         if type(a) != type(b):
             return False
-        if isinstance(a, dict):
-            if not dict_extra and set(a.keys()) != set(b.keys()):
-                return False
-            return all(json_compare(a[k],
-                                    b[k],
+        return a == b
+
+    if isinstance(a, (SupportsFloat, SupportsInt)) and isinstance(b, (SupportsFloat, SupportsInt)):
+        isint_a = isinstance(a, int) or not isinstance(a, SupportsFloat)
+        isint_b = isinstance(b, int) or not isinstance(b, SupportsFloat)
+        if isint_a and isint_b:
+            return int(a) == int(b)
+        if int_strict and (isint_a or isint_b):
+            return False
+        va = int(a) if isint_a else float(a)
+        vb = int(b) if isint_b else float(b)
+        if math.isnan(va) and math.isnan(vb):
+            return True
+        if math.isinf(va) and math.isinf(vb):
+            return va == vb
+        return abs(va - vb) <= float_tol
+
+    if isinstance(a, Mapping) and isinstance(b, Mapping):
+        if not dict_extra and set(a.keys()) != set(b.keys()):
+            return False
+        return all(json_compare(a[k],
+                                b[k],
+                                int_strict=int_strict,
+                                float_tol=float_tol,
+                                list_order=list_order,
+                                dict_extra=dict_extra)
+                   for k in set(a.keys()) & set(b.keys()))
+
+    if isinstance(a, Sequence) and isinstance(b, Sequence):
+        if len(a) != len(b):
+            return False
+        if list_order:
+            return all(json_compare(va,
+                                    vb,
                                     int_strict=int_strict,
                                     float_tol=float_tol,
                                     list_order=list_order,
                                     dict_extra=dict_extra)
-                       for k in set(a.keys()) & set(b.keys()))
-        if isinstance(a, list):
-            if len(a) != len(b):
-                return False
-            if list_order:
-                return all(json_compare(va,
-                                        vb,
-                                        int_strict=int_strict,
-                                        float_tol=float_tol,
-                                        list_order=list_order,
-                                        dict_extra=dict_extra)
-                           for va, vb in zip(a, b))
-            else:
-                return all(json_compare(va,
-                                        vb,
-                                        int_strict=int_strict,
-                                        float_tol=float_tol,
-                                        list_order=list_order,
-                                        dict_extra=dict_extra)
-                           for va, vb in zip(sorted(a), sorted(b)))
-        return a == b
-    if isinstance(a, (float, int)) and isinstance(b, (float, int)):
-        if int_strict:
-            if type(a) != type(b):
-                return False
-        if isinstance(a, float) or isinstance(b, float):
-            if math.isnan(a) and math.isnan(b):
-                return True
-            if math.isinf(a) and math.isinf(b):
-                return a == b
-            return abs(a - b) <= float_tol
-        return a == b
+                       for va, vb in zip(a, b))
+        else:
+            return all(json_compare(va,
+                                    vb,
+                                    int_strict=int_strict,
+                                    float_tol=float_tol,
+                                    list_order=list_order,
+                                    dict_extra=dict_extra)
+                       for va, vb in zip(sorted(a), sorted(b)))
+
     raise ValueError(f"incompatible type '{type(a)}' and '{type(b)}'")
