@@ -1,3 +1,4 @@
+import itertools
 import unittest
 
 import ddt
@@ -130,3 +131,39 @@ class RandUtilsTest(unittest.TestCase):
             result = randomizer().random_time(begin, end)
 
             self.assertTrue(dt_utc_min().timetz() <= begin <= result <= end <= dt_utc_max().timetz())
+
+    def test_sample(self):
+        for _ in range(1000):
+            population = list(itertools.accumulate(randomizer().next_int(1, 10)
+                                                   for _ in range(randomizer().next_int(100, 200))))
+            count_func = lambda x: 1 + x % 3
+            k = randomizer().next_int(1, len(population))
+
+            result = randomizer().sample(population, count_func, k)
+
+            self.assertTrue(len(result), k)
+            self.assertTrue(all(elem in population for elem in result))
+            self.assertTrue(all(result.count(elem) <= count_func(elem) for elem in result))
+
+    def test_choose(self):
+        for _ in range(1000):
+            population = list(itertools.accumulate(randomizer().next_int(1, 10)
+                                                   for _ in range(randomizer().next_int(100, 200))))
+            weight_func = lambda x: float(1 + x % 3)
+            k = randomizer().next_int(1, len(population))
+
+            result = randomizer().choose(population, weight_func, k)
+
+            self.assertTrue(len(result), k)
+            self.assertTrue(all(elem in population for elem in result))
+
+    def test_shuffle(self):
+        for _ in range(1000):
+            data = list(itertools.accumulate(randomizer().next_int(1, 10)
+                                             for _ in range(randomizer().next_int(100, 200))))
+
+            result = randomizer().shuffle(data)
+
+            self.assertIsNot(result, data)
+            self.assertTrue(len(result), len(data))
+            self.assertTrue(all(elem in data for elem in result))
