@@ -1,6 +1,7 @@
 import unittest
 
 import ddt
+import pytest
 
 from iker.common.utils.sequtils import batch_yield
 from iker.common.utils.sequtils import chunk, chunk_between, chunk_with_key, merge_chunks
@@ -789,6 +790,43 @@ class SeqUtilsTest(unittest.TestCase):
     def test_merge_chunks__drop_exclusive_end(self, data, merge_func, expect):
         self.assertEqual(expect, merge_chunks(data, merge_func=merge_func, drop_exclusive_end=True))
 
+
+@ddt.ddt
+class SeqTest(unittest.TestCase):
+
+    @ddt.data(
+        ([], []),
+        (seq([]), []),
+        (seq(seq([])), []),
+        ([0], [0]),
+        (seq([0]), [0]),
+        (seq(seq([0])), [0]),
+        ([1, 2, 3, 4, 5], [1, 2, 3, 4, 5]),
+        (seq([1, 2, 3, 4, 5]), [1, 2, 3, 4, 5]),
+        (seq(seq([1, 2, 3, 4, 5])), [1, 2, 3, 4, 5]),
+        (range(1, 6), [1, 2, 3, 4, 5]),
+        (seq(range(1, 6)), [1, 2, 3, 4, 5]),
+        (seq(seq(range(1, 6))), [1, 2, 3, 4, 5]),
+    )
+    @ddt.unpack
+    def test_builtin_init(self, data, expect):
+        self.assertEqual(seq(data).data, expect)
+
+    def test_builtin_init__unsupported_data_type(self):
+        with pytest.raises(ValueError):
+            seq(object())
+
+    @ddt.data(
+        ([], [], []),
+        ([0], [], [0]),
+        ([], [0], [0]),
+        ([1, 2, 3, 4, 5], [1, 2, 3, 4, 5], [1, 2, 3, 4, 5, 1, 2, 3, 4, 5]),
+    )
+    @ddt.unpack
+    def test_builtin_add(self, a, b, expect):
+        actual = seq(a) + seq(b)
+        self.assertEqual(actual.data, expect)
+
     @ddt.data(
         ([0], 0, [0]),
         ([1, 2, 3, 4, 5], 0, [1]),
@@ -801,17 +839,21 @@ class SeqUtilsTest(unittest.TestCase):
         ([1, 2, 3, 4, 5], slice(1, -1, 2), [2, 4]),
     )
     @ddt.unpack
-    def test_seq_builtin_getitem(self, data, item, expect):
+    def test_builtin_getitem(self, data, item, expect):
         actual = seq(data)[item]
         self.assertEqual(actual.data, expect)
 
+    def test_builtin_getitem__unsupported_index_type(self):
+        with pytest.raises(ValueError):
+            seq([])[object()]
+
     @ddt.data(
         ([], 0),
         ([0], 1),
         ([1, 2, 3, 4, 5], 5),
     )
     @ddt.unpack
-    def test_seq_builtin_len(self, data, expect):
+    def test_builtin_len(self, data, expect):
         actual = len(seq(data))
         self.assertEqual(actual, expect)
 
@@ -824,7 +866,7 @@ class SeqUtilsTest(unittest.TestCase):
         ([1, 2, 3, 4, 5], -1, False),
     )
     @ddt.unpack
-    def test_seq_builtin_contains(self, data, item, expect):
+    def test_builtin_contains(self, data, item, expect):
         actual = item in seq(data)
         self.assertEqual(actual, expect)
 
@@ -834,7 +876,7 @@ class SeqUtilsTest(unittest.TestCase):
         ([1, 2, 3, 4, 5], [1, 2, 3, 4, 5]),
     )
     @ddt.unpack
-    def test_seq_builtin_iter(self, data, expect):
+    def test_builtin_iter(self, data, expect):
         actual = list(x for x in seq(data))
         self.assertEqual(actual, expect)
 
@@ -844,7 +886,7 @@ class SeqUtilsTest(unittest.TestCase):
         ([1, 2, 3, 4, 5], [5, 4, 3, 2, 1]),
     )
     @ddt.unpack
-    def test_seq_builtin_reversed(self, data, expect):
+    def test_builtin_reversed(self, data, expect):
         actual = list(x for x in reversed(seq(data)))
         self.assertEqual(actual, expect)
 
@@ -859,7 +901,7 @@ class SeqUtilsTest(unittest.TestCase):
         ([1, 2, 3, 4, 5], -1, 0),
     )
     @ddt.unpack
-    def test_seq_count(self, data, item, expect):
+    def test_count(self, data, item, expect):
         actual = seq(data).count(item)
         self.assertEqual(actual, expect)
 
@@ -879,10 +921,65 @@ class SeqUtilsTest(unittest.TestCase):
         ([1, 2, 2, 3, 3, 3, 4, 4, 4, 4, 5, 5, 5, 5, 5], lambda x: x % 2 == 1, 9),
     )
     @ddt.unpack
-    def test_seq_count_if(self, data, func, expect):
+    def test_count_if(self, data, func, expect):
         actual = seq(data).count_if(func)
         self.assertEqual(actual, expect)
 
+    @ddt.data(
+        ([], 0, []),
+        ([], 1, []),
+        ([0], 0, []),
+        ([0], 1, [0]),
+        ([0], 2, [0]),
+        ([1, 2, 3, 4, 5], 0, []),
+        ([1, 2, 3, 4, 5], 1, [1]),
+        ([1, 2, 3, 4, 5], 5, [1, 2, 3, 4, 5]),
+        ([1, 2, 3, 4, 5], 6, [1, 2, 3, 4, 5]),
+    )
+    @ddt.unpack
+    def test_take_left(self, data, n, expect):
+        actual = seq(data).take_left(n)
+        self.assertEqual(actual.data, expect)
+
+    @ddt.data(
+        ([], 0, []),
+        ([], 1, []),
+        ([0], 0, []),
+        ([0], 1, [0]),
+        ([0], 2, [0]),
+        ([1, 2, 3, 4, 5], 0, []),
+        ([1, 2, 3, 4, 5], 1, [5]),
+        ([1, 2, 3, 4, 5], 5, [1, 2, 3, 4, 5]),
+        ([1, 2, 3, 4, 5], 6, [1, 2, 3, 4, 5]),
+    )
+    @ddt.unpack
+    def test_take_right(self, data, n, expect):
+        actual = seq(data).take_right(n)
+        self.assertEqual(actual.data, expect)
+
+    @ddt.data(
+        ([], []),
+        ([0], [0]),
+        ([0, 0, 0], [0, 0, 0]),
+        ([1, 2, 3, 4, 5], [5, 4, 3, 2, 1]),
+    )
+    @ddt.unpack
+    def test_reverse(self, data, expect):
+        actual = seq(data).reverse()
+        self.assertEqual(actual.data, expect)
+
+    @ddt.data(
+        ([], []),
+        ([0], [0]),
+        ([0, 0, 0], [0]),
+        ([1, 2, 3, 4, 5], [1, 2, 3, 4, 5]),
+        ([1, 2, 3, 4, 5, 1, 2, 3, 4, 1, 2, 3, 1, 2, 1], [1, 2, 3, 4, 5]),
+    )
+    @ddt.unpack
+    def test_distinct(self, data, expect):
+        actual = seq(data).distinct()
+        self.assertEqual(actual.data, expect)
+
     @ddt.data(
         ([], 0, lambda x, y: x, []),
         ([], None, lambda x, y: x, []),
@@ -892,7 +989,7 @@ class SeqUtilsTest(unittest.TestCase):
         ([1, 2, 3, 4, 5], 0, lambda x, y: x * 10 + y, [1, 12, 123, 1234, 12345]),
     )
     @ddt.unpack
-    def test_seq_scan_left(self, data, zero, func, expect):
+    def test_scan_left(self, data, zero, func, expect):
         actual = seq(data).scan_left(zero, func)
         self.assertEqual(actual.data, expect)
 
@@ -905,7 +1002,7 @@ class SeqUtilsTest(unittest.TestCase):
         ([1, 2, 3, 4, 5], 0, lambda x, y: x * 10 + y, [54321, 5432, 543, 54, 5]),
     )
     @ddt.unpack
-    def test_seq_scan_right(self, data, zero, func, expect):
+    def test_scan_right(self, data, zero, func, expect):
         actual = seq(data).scan_right(zero, func)
         self.assertEqual(actual.data, expect)
 
@@ -916,7 +1013,7 @@ class SeqUtilsTest(unittest.TestCase):
         ([1, 2, 3, 4, 5], lambda x: x ** 2, [1, 4, 9, 16, 25]),
     )
     @ddt.unpack
-    def test_seq_map(self, data, func, expect):
+    def test_map(self, data, func, expect):
         actual = seq(data).map(func)
         self.assertEqual(actual.data, expect)
 
@@ -929,7 +1026,7 @@ class SeqUtilsTest(unittest.TestCase):
         ([1, 2, 3, 4, 5], 0, lambda x, y: x * 10 + y, [12345]),
     )
     @ddt.unpack
-    def test_seq_fold_left(self, data, zero, func, expect):
+    def test_fold_left(self, data, zero, func, expect):
         actual = seq(data).fold_left(zero, func)
         self.assertEqual(actual.data, expect)
 
@@ -941,7 +1038,7 @@ class SeqUtilsTest(unittest.TestCase):
         ([1, 2, 3, 4, 5], 0, lambda x, y: x * 10 + y, [54321]),
     )
     @ddt.unpack
-    def test_seq_fold_right(self, data, zero, func, expect):
+    def test_fold_right(self, data, zero, func, expect):
         actual = seq(data).fold_right(zero, func)
         self.assertEqual(actual.data, expect)
 
@@ -952,7 +1049,7 @@ class SeqUtilsTest(unittest.TestCase):
         ([1, 2, 3, 4, 5], lambda x, y: y, [5]),
     )
     @ddt.unpack
-    def test_seq_reduce(self, data, func, expect):
+    def test_reduce(self, data, func, expect):
         actual = seq(data).reduce(func)
         self.assertEqual(actual.data, expect)
 
@@ -964,7 +1061,7 @@ class SeqUtilsTest(unittest.TestCase):
         ([1, 2, 3, 4, 5], lambda x, y: False, [5]),
     )
     @ddt.unpack
-    def test_seq_max(self, data, func, expect):
+    def test_max(self, data, func, expect):
         actual = seq(data).max(func)
         self.assertEqual(actual.data, expect)
 
@@ -976,7 +1073,7 @@ class SeqUtilsTest(unittest.TestCase):
         ([1, 2, 3, 4, 5], lambda x, y: False, [5]),
     )
     @ddt.unpack
-    def test_seq_min(self, data, func, expect):
+    def test_min(self, data, func, expect):
         actual = seq(data).min(func)
         self.assertEqual(actual.data, expect)
 
@@ -987,7 +1084,7 @@ class SeqUtilsTest(unittest.TestCase):
         ([1, 2, 3, 4, 5], lambda x: x * 2, [(2, [1]), (4, [2]), (6, [3]), (8, [4]), (10, [5])]),
     )
     @ddt.unpack
-    def test_seq_group(self, data, func, expect):
+    def test_group(self, data, func, expect):
         actual = seq(data).group(func)
         self.assertEqual(actual.data, expect)
 
@@ -997,7 +1094,7 @@ class SeqUtilsTest(unittest.TestCase):
         ([(1, "a"), (2, "b"), (3, "c"), (4, "d"), (5, "e")], [1, 2, 3, 4, 5]),
     )
     @ddt.unpack
-    def test_seq_keys(self, data, expect):
+    def test_keys(self, data, expect):
         actual = seq(data).keys()
         self.assertEqual(actual.data, expect)
 
@@ -1007,7 +1104,7 @@ class SeqUtilsTest(unittest.TestCase):
         ([(1, "a"), (2, "b"), (3, "c"), (4, "d"), (5, "e")], ["a", "b", "c", "d", "e"]),
     )
     @ddt.unpack
-    def test_seq_values(self, data, expect):
+    def test_values(self, data, expect):
         actual = seq(data).values()
         self.assertEqual(actual.data, expect)
 
@@ -1017,10 +1114,64 @@ class SeqUtilsTest(unittest.TestCase):
         ([(1, "a"), (2, "b"), (3, "c"), (4, "d"), (5, "e")], [("a", 1), ("b", 2), ("c", 3), ("d", 4), ("e", 5)]),
     )
     @ddt.unpack
-    def test_seq_swap(self, data, expect):
+    def test_swap(self, data, expect):
         actual = seq(data).swap()
         self.assertEqual(actual.data, expect)
 
+    @ddt.data(
+        ([], lambda x: x, []),
+        ([(0, "0")], lambda x: x * 2, [(0, "0")]),
+        ([(0, "0")], str, [("0", "0")]),
+        (
+            [(1, "1"), (2, "2"), (3, "3"), (4, "4"), (5, "5")],
+            lambda x: x * 2,
+            [(2, "1"), (4, "2"), (6, "3"), (8, "4"), (10, "5")],
+        ),
+        (
+            [(1, "1"), (2, "2"), (3, "3"), (4, "4"), (5, "5")],
+            str,
+            [("1", "1"), ("2", "2"), ("3", "3"), ("4", "4"), ("5", "5")],
+        ),
+    )
+    @ddt.unpack
+    def test_map_keys(self, data, func, expect):
+        actual = seq(data).map_keys(func)
+        self.assertEqual(actual.data, expect)
+
+    @ddt.data(
+        ([], lambda x: x, []),
+        ([(0, "0")], lambda x: x * 2, [(0, "00")]),
+        ([(0, "0")], int, [(0, 0)]),
+        (
+            [(1, "1"), (2, "2"), (3, "3"), (4, "4"), (5, "5")],
+            lambda x: x * 2,
+            [(1, "11"), (2, "22"), (3, "33"), (4, "44"), (5, "55")],
+        ),
+        (
+            [(1, "1"), (2, "2"), (3, "3"), (4, "4"), (5, "5")],
+            int,
+            [(1, 1), (2, 2), (3, 3), (4, 4), (5, 5)],
+        ),
+    )
+    @ddt.unpack
+    def test_map_values(self, data, func, expect):
+        actual = seq(data).map_values(func)
+        self.assertEqual(actual.data, expect)
+
+    @ddt.data(
+        ([], lambda x: x, []),
+        ([[]], lambda x: x, []),
+        ([[0]], lambda x: x, [0]),
+        ([[1, 2, 3, 4, 5]], lambda x: x, [1, 2, 3, 4, 5]),
+        ([[], [1], [2], [3], [4], [5], []], lambda x: x, [1, 2, 3, 4, 5]),
+        ([[], [1, 2, 3, 4, 5], []], lambda x: x, [1, 2, 3, 4, 5]),
+        ([1, 2, 3, 4, 5], lambda x: list(range(x)), [0, 0, 1, 0, 1, 2, 0, 1, 2, 3, 0, 1, 2, 3, 4]),
+    )
+    @ddt.unpack
+    def test_flat_map(self, data, func, expect):
+        actual = seq(data).flat_map(func)
+        self.assertEqual(actual.data, expect)
+
     @ddt.data(
         ([], []),
         ([[]], []),
@@ -1030,10 +1181,76 @@ class SeqUtilsTest(unittest.TestCase):
         ([[], [1, 2, 3, 4, 5], []], [1, 2, 3, 4, 5]),
     )
     @ddt.unpack
-    def test_seq_flatten(self, data, expect):
+    def test_flatten(self, data, expect):
         actual = seq(data).flatten()
         self.assertEqual(actual.data, expect)
 
+    @ddt.data(
+        ([], lambda x: x % 2, lambda x: x ** 2, []),
+        ([0], lambda x: x % 2, lambda x: x ** 2, [(0, [0])]),
+        ([1, 2, 3, 4, 5], lambda x: x % 2, lambda x: x ** 2, [(0, [4, 16]), (1, [1, 9, 25])]),
+        ([1, 2, 3, 4, 5], lambda x: x, lambda x: x ** 2, [(1, [1]), (2, [4]), (3, [9]), (4, [16]), (5, [25])]),
+    )
+    @ddt.unpack
+    def test_group_map(self, data, group_func, map_func, expect):
+        actual = seq(data).group_map(group_func, map_func)
+        self.assertEqual(actual.data, expect)
+
+    @ddt.data(
+        ([], lambda x: True, []),
+        ([0], lambda x: True, [0]),
+        ([0], lambda x: False, []),
+        ([0], lambda x: x % 2 == 0, [0]),
+        ([0], lambda x: x % 2 == 1, []),
+        ([0, 0, 0], lambda x: True, [0, 0, 0]),
+        ([0, 0, 0], lambda x: False, []),
+        ([0, 0, 0], lambda x: x % 2 == 0, [0, 0, 0]),
+        ([0, 0, 0], lambda x: x % 2 == 1, []),
+        ([1, 2, 3, 4, 5], lambda x: x % 2 == 0, [2, 4]),
+        ([1, 2, 3, 4, 5], lambda x: x % 2 == 1, [1, 3, 5]),
+        ([1, 2, 2, 3, 3, 3, 4, 4, 4, 4, 5, 5, 5, 5, 5], lambda x: x % 2 == 0, [2, 2, 4, 4, 4, 4]),
+        ([1, 2, 2, 3, 3, 3, 4, 4, 4, 4, 5, 5, 5, 5, 5], lambda x: x % 2 == 1, [1, 3, 3, 3, 5, 5, 5, 5, 5]),
+    )
+    @ddt.unpack
+    def test_filter(self, data, func, expect):
+        actual = seq(data).filter(func)
+        self.assertEqual(actual.data, expect)
+
+    @ddt.data(
+        ([], lambda x: True, []),
+        ([0], lambda x: True, []),
+        ([0], lambda x: False, [0]),
+        ([0], lambda x: x % 2 == 0, []),
+        ([0], lambda x: x % 2 == 1, [0]),
+        ([0, 0, 0], lambda x: True, []),
+        ([0, 0, 0], lambda x: False, [0, 0, 0]),
+        ([0, 0, 0], lambda x: x % 2 == 0, []),
+        ([0, 0, 0], lambda x: x % 2 == 1, [0, 0, 0]),
+        ([1, 2, 3, 4, 5], lambda x: x % 2 == 0, [1, 3, 5]),
+        ([1, 2, 3, 4, 5], lambda x: x % 2 == 1, [2, 4]),
+        ([1, 2, 2, 3, 3, 3, 4, 4, 4, 4, 5, 5, 5, 5, 5], lambda x: x % 2 == 0, [1, 3, 3, 3, 5, 5, 5, 5, 5]),
+        ([1, 2, 2, 3, 3, 3, 4, 4, 4, 4, 5, 5, 5, 5, 5], lambda x: x % 2 == 1, [2, 2, 4, 4, 4, 4]),
+    )
+    @ddt.unpack
+    def test_filter_not(self, data, func, expect):
+        actual = seq(data).filter_not(func)
+        self.assertEqual(actual.data, expect)
+
+    @ddt.data(
+        ([], lambda x: x, []),
+        ([0], lambda x: x, [0]),
+        ([0, 0, 0], lambda x: x, [0, 0, 0]),
+        ([1, 2, 3, 4, 5], lambda x: x, [1, 2, 3, 4, 5]),
+        ([5, 4, 3, 2, 1], lambda x: x, [1, 2, 3, 4, 5]),
+        ([1, 2, 2, 3, 3, 3, 4, 4, 4, 4, 5, 5, 5, 5, 5], lambda x: x, [1, 2, 2, 3, 3, 3, 4, 4, 4, 4, 5, 5, 5, 5, 5]),
+        ([1, 2, 3, 4, 5, 2, 3, 4, 5, 3, 4, 5, 4, 5, 5], lambda x: x, [1, 2, 2, 3, 3, 3, 4, 4, 4, 4, 5, 5, 5, 5, 5]),
+        ([1, 2, 3, 4, 5, 2, 3, 4, 5, 3, 4, 5, 4, 5, 5], lambda x: -x, [5, 5, 5, 5, 5, 4, 4, 4, 4, 3, 3, 3, 2, 2, 1]),
+    )
+    @ddt.unpack
+    def test_sort(self, data, func, expect):
+        actual = (seq(data).sort(func))
+        self.assertEqual(actual.data, expect)
+
     @ddt.data(
         ([], [], []),
         ([0], [], []),
@@ -1047,7 +1264,7 @@ class SeqUtilsTest(unittest.TestCase):
         (["a", "b", "c", "d", "e"], [1, 2, 3, 4, 5], [("a", 1), ("b", 2), ("c", 3), ("d", 4), ("e", 5)]),
     )
     @ddt.unpack
-    def test_seq_zip(self, a, b, expect):
+    def test_zip(self, a, b, expect):
         actual = seq(a).zip(seq(b))
         self.assertEqual(actual.data, expect)
 
@@ -1096,6 +1313,6 @@ class SeqUtilsTest(unittest.TestCase):
         ),
     )
     @ddt.unpack
-    def test_seq_zip_fill(self, a, b, fill, expect):
+    def test_zip_fill(self, a, b, fill, expect):
         actual = seq(a).zip_fill(seq(b), fill)
         self.assertEqual(actual.data, expect)
